@@ -552,6 +552,8 @@ def processing_multi(**opt_args):
     with closing(Pool(processes=args["max_processes"])) as pp:
         job_list = [pp.apply_async(func=ac_for_line, args=(i1,), kwds=opt_args) for i1 in range(*args["subset_sample"])]
         tqdm_joblist(job_list, n_status_prints=100, apply_on_done=copy_results)
+        pp.close()
+        pp.join()
     print("")
 
 
