@@ -1315,6 +1315,8 @@ def __minimize__(pt_index, p0, pt_names, data, opt_func, zoom_factor=None, opt_r
                 p_bar.update()
                 p_bar.cnt -= 1
                 sleep(1)
+            pl.close()
+            pl.join()
     else:
         if debug is True:
             initializer(globals(), globs)
@@ -1323,6 +1325,8 @@ def __minimize__(pt_index, p0, pt_names, data, opt_func, zoom_factor=None, opt_r
         else:
             with Pool(processes=processes, initializer=initializer, initargs=(globals(), globs,)) as pl:
                 pl.map(__min__, rng)
+                pl.close()
+                pl.join()
 
     for ii, (up, dn) in enumerate(zip(opt_func.x_bounds_up, opt_func.x_bounds_dn)):
         globs["__res__"].np[globs["__res__"].np[:, :, ii] > up, ii] = up
