@@ -46,7 +46,7 @@ from sicor.Tools.EnMAP.LUT import reduce_lut, interpol_lut_red
 from sicor.Tools.EnMAP.conversion import generate_filter, table_to_array
 from sicor.Tools.EnMAP.optimal_estimation import invert_function
 from sicor.Tools.EnMAP.multiprocessing import SharedNdarray, initializer, mp_progress_bar
-from sicor.Tools.EnMAP.a_priori import wv_band_ratio
+from sicor.Tools.EnMAP.first_guess import wv_band_ratio
 from sicor.Tools.EnMAP.segmentation import SLIC_segmentation
 
 
@@ -1258,6 +1258,8 @@ def __minimize__(fo, opt_func, unknowns=False, logger=None):
                 if results.ready():
                     results.get()
                     break
+            pl.close()
+            pl.join()
     t1 = time()
 
     cwv_model = globs["__cwv_model__"].np
