@@ -12,11 +12,9 @@ import logging
 from datetime import timedelta
 from os.path import join, dirname, isfile
 from os import makedirs, remove
-# non-standard imports
 from netCDF4 import Dataset
 from multiprocessing import Pool
 
-
 __author__ = "Niklas Bohn, Andre Hollstein"
 
 """
@@ -27,6 +25,14 @@ https://software.ecmwf.int/wiki/display/WEBAPI/Accessing+ECMWF+data+servers+in+b
 keyfile: needed in $HOME/.ecmwfapirc
 """
 
+"""
+From July 01, 2021, 'cams_nrealtime' datasets have to be accessed via the atmosphere data store api. The respective
+dataset is now named 'cams-global-atmospheric-composition-forecasts'.
+
+The user needs to register at https://ads.atmosphere.copernicus.eu, to save UID:key in .cdsapirc file, and to accept
+licenses and terms of use. The download is based on the cdsapi python package (https://pypi.org/project/cdsapi/).
+"""
+
 """
 pip install pygrib # might not work for you since it did not work for me -> manual install might be needed here
 1st: grib_api
@@ -65,61 +71,40 @@ grib_api_dir = /home/andre/working/enmap_cloud_sceening/momo/get_ECMWF_products/
 
 # noinspection PyPep8
 class ECMWF_download(object):
-    def __init__(self, server, logger=None, max_step=120):
+    def __init__(self, server, client, logger=None, max_step=120):
         """
         Class for downloading data from the ECMWF service.
         :param server: instance of ECMWFDataServer
+        :param client: instance of cdsapi.Client
         """
+
         self.logger = logger or logging.getLogger(__name__)
         self.server = server
-
-        # "hard wired" parameters for retrieving data from era interim
-        # get for analysis: http://apps.ecmwf.int/datasets/data/interim-full-daily/
-        # get for forecast: http://apps.ecmwf.int/datasets/data/cams-nrealtime/
+        self.client = client
 
         ecmwf_fc_steps = [0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48, 51, 54, 57, 60, 63, 66, 69,
                           72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 105, 108, 111, 114, 117, 120]
-        step_fc = "/".join([str(step) for step in ecmwf_fc_steps if step <= max_step])
+        step_fc_ecmwf = "/".join([str(step) for step in ecmwf_fc_steps if step <= max_step])
+        step_fc_cds = [str(step) for step in ecmwf_fc_steps if step <= max_step]
 
         self.era_setup = {
             "fc_T2M": {
-                "Name": "2 metre temperature",
-                "class": "mc",
-                "dataset": "cams_nrealtime",
-                "expver": "0001",
-                "levtype": "sfc",
-                "param": "167.128",
-                "step": step_fc,
-                "stream": "oper",
-                "time": "00:00:00",
-                "type": "fc",
-                "levellist": None
+                "variable": "2m_temperature",
+                "time": ["00:00", "12:00"],
+                "type": "forecast",
+                "leadtime_hour": step_fc_cds
             },
             "fc_O3": {
-                "Name": "GEMS Total column ozone",
-                "class": "mc",
-                "dataset": "cams_nrealtime",
-                "expver": "0001",
-                "levtype": "sfc",
-                "param": "206.210",
-                "step": step_fc,
-                "stream": "oper",
-                "time": "00:00:00",
-                "type": "fc",
-                "levellist": None
+                "variable": "total_column_ozone",
+                "time": ["00:00", "12:00"],
+                "type": "forecast",
+                "leadtime_hour": step_fc_cds
             },
             "fc_SLP": {
-                "Name": "Mean sea level pressure",
-                "class": "mc",
-                "dataset": "cams_nrealtime",
-                "expver": "0001",
-                "levtype": "sfc",
-                "param": "151.128",
-                "step": step_fc,
-                "stream": "oper",
-                "time": "00:00:00",
-                "type": "fc",
-                "levellist": None
+                "variable": "surface_pressure",
+                "time": ["00:00", "12:00"],
+                "type": "forecast",
+                "leadtime_hour": step_fc_cds
             },
             "an_TCWV": {  # untested
                 "Name": "Total column water vapour",
@@ -142,7 +127,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "137.128",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -155,7 +140,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "206.210",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -183,7 +168,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "207.210",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -196,7 +181,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "212.210",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -209,7 +194,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "211.210",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -222,7 +207,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "209.210",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -235,7 +220,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "210.210",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -248,7 +233,7 @@ class ECMWF_download(object):
                 "expver": "0001",
                 "levtype": "sfc",
                 "param": "208.210",
-                "step": step_fc,
+                "step": step_fc_ecmwf,
                 "stream": "oper",
                 "time": "00:00:00",
                 "type": "fc",
@@ -360,17 +345,19 @@ class ECMWF_download(object):
         return join(db_path, variable, str(dt.year), "%02i" % dt.month,
                     "%4i%02i%02i_%s.h5" % (dt.year, dt.month, dt.day, variable))
 
-    def era_month_string(self, year, month=None, month_start=None, month_end=None, day_start=None, day_end=None):
+    def era_month_string(self, year, month=None, month_start=None, month_end=None, day_start=None, day_end=None,
+                         client=None):
         """
-        Create string for ERA request
+        Create string for ERA request.
 
-        :param year: integer, year
-        :param month: integer, month or None, then month_start and month_end should be given
+        :param year:        integer, year
+        :param month:       integer, month or None, if None, month_start and month_end should be given
         :param month_start: integer or None, if None, month should be given
-        :param month_end: integer or None, if None, month should be given
-        :param day_start: integer or None, if None, assume 1
-        :param day_end: integer or None, if None, end of month is used
-        :return: string for era request
+        :param month_end:   integer or None, if None, month should be given
+        :param day_start:   integer or None, if None, assume 1
+        :param day_end:     integer or None, if None, end of month is used
+        :param client:      'ecmwf' for 'macc_nrealtime' datasets or 'cds' for 'cams_nrealtime' datasets
+        :return:            string for era request
         """
 
         # basic sanity check for date inputs
@@ -409,29 +396,38 @@ class ECMWF_download(object):
         self.tind_start = self.time_to_ind(self.jday_start, 0)
         self.tind_end = self.time_to_ind(self.jday_end, 3)
 
-        return "%i-%0.2i-%0.2i/to/%i-%0.2i-%0.2i" % (year, month_start, day_start, year, month_end, day_end)
+        if client == 'cds':
+            return "%i-%0.2i-%0.2i/%i-%0.2i-%0.2i" % (year, month_start, day_start, year, month_end, day_end)
+        else:
+            return "%i-%0.2i-%0.2i/to/%i-%0.2i-%0.2i" % (year, month_start, day_start, year, month_end, day_end)
 
     def request(self, parameter, output_file_name, year, month=None, month_start=None, month_end=None, day_start=None,
                 day_end=None, nc=False):
         """
-        Prepare request for ECMWF server
-        :param parameter: as given in self.era_setup
-        :param output_file_name: string, file name, should be grb
-        :param year:
-        :param month:
-        :param month_start:
-        :param month_end:
-        :param day_start:
-        :param day_end:
-        :param nc:
-        :return:
-        """
-        assert parameter in self.era_setup, "parameter should one of:" + str(list(self.era_setup.keys()))
+        Prepare request for ECMWF server.
 
-        rq = {key: item for key, item in self.era_setup[parameter].items() if key in
-              ["class", "dataset", "date", "expver", "levtype", "param", "step", "stream", "time", "type"]}
-        rq["target"] = output_file_name
-        rq["date"] = self.era_month_string(year, month, month_start, month_end, day_start, day_end)
+        :param parameter:        as given in self.era_setup
+        :param output_file_name: string, file name, should be grb
+        :param year:             integer, year
+        :param month:            integer, month or None, if None, month_start and month_end should be given
+        :param month_start:      integer or None, if None, month should be given
+        :param month_end:        integer or None, if None, month should be given
+        :param day_start:        integer or None, if None, assume 1
+        :param day_end:          integer or None, if None, end of month is used
+        :param nc:               if True, set format 'netcdf'; default: False
+        :return:                 dict, request for ECMWF server
+        """
+        assert parameter in self.era_setup, "parameter should be one of:" + str(list(self.era_setup.keys()))
+
+        if parameter in ["fc_T2M", "fc_O3", "fc_SLP"]:
+            rq = {key: item for key, item in self.era_setup[parameter].items() if key in
+                  ["variable", "time", "type", "leadtime_hour"]}
+            rq["date"] = self.era_month_string(year, month, month_start, month_end, day_start, day_end, client='cds')
+        else:
+            rq = {key: item for key, item in self.era_setup[parameter].items() if key in
+                  ["class", "dataset", "date", "expver", "levtype", "param", "step", "stream", "time", "type"]}
+            rq["target"] = output_file_name
+            rq["date"] = self.era_month_string(year, month, month_start, month_end, day_start, day_end, client='ecmwf')
         if nc:
             rq["format"] = "netcdf"
             rq["grid"] = "0.705/0.705"
@@ -455,11 +451,19 @@ class ECMWF_download(object):
 
     def retrieve_grib(self, **kwargs):
         req = self.request(**kwargs)
-        self.server.retrieve(req)
+        if kwargs["parameter"] in ["fc_T2M", "fc_O3", "fc_SLP"]:
+            self.client.retrieve(name='cams-global-atmospheric-composition-forecasts', request=req,
+                                 target=kwargs["output_file_name"])
+        else:
+            self.server.retrieve(req)
 
     def retrieve_netcdf(self, **kwargs):
         req = self.request(nc=True, **kwargs)
-        self.server.retrieve(req)
+        if kwargs["parameter"] in ["fc_T2M", "fc_O3", "fc_SLP"]:
+            self.client.retrieve(name='cams-global-atmospheric-composition-forecasts', request=req,
+                                 target=kwargs["output_file_name"])
+        else:
+            self.server.retrieve(req)
 
     def retrieve_hdf5(self, complevel=1, remove_ECMWF_file=True, grib=True, **kwargs):
         hdf_file = kwargs["output_file_name"]
@@ -499,13 +503,23 @@ class ECMWF_download(object):
             lats, lons = era_data_file.message(1).latlons()
             lat, lon = lats[:, 0], lons[0, :]
 
-            if self.era_setup[parameter]["step"] != "0":  # we use forcast data
+            if parameter in ["fc_T2M", "fc_O3", "fc_SLP"]:
+                step = self.era_setup[parameter]["leadtime_hour"]
+            else:
+                step = self.era_setup[parameter]["step"]
+            if step != "0":  # we use forcast data
                 tp = "forcast"
                 tinds = np.arange(self.tind_start, self.tind_end + 1)  # four times per day for reanalysis
                 # number of steps * number of days
-                n_time = int((self.era_setup[parameter]["step"].count("/") + 1)) * int((len(tinds) / 4))
+                if parameter in ["fc_T2M", "fc_O3", "fc_SLP"]:
+                    n_time = len(step) * int((len(tinds) / 4))
+                else:
+                    n_time = int((step.count("/") + 1)) * int((len(tinds) / 4))
                 tinds = tinds[::4]  # only t0
-                steps = [int(s) for s in self.era_setup[parameter]["step"].split("/")]
+                if parameter in ["fc_T2M", "fc_O3", "fc_SLP"]:
+                    steps = [int(s) for s in step]
+                else:
+                    steps = [int(s) for s in step.split("/")]
             else:  # we use reanalysis data, 4 steps per day
                 tp = "reanalysis"
                 tinds = np.arange(self.tind_start, self.tind_end + 1)
@@ -523,8 +537,8 @@ class ECMWF_download(object):
             self.logger.info("Create HDF file: %s" % hdf_file)
             with tables.open_file(hdf_file, "w") as era_h5_file:
                 era_h5 = era_h5_file.root
-
-                if self.era_setup[parameter]['levellist'] is None:  # 2D type parameter
+                if 'levellist' not in self.era_setup[parameter].keys() or not self.era_setup[parameter]['levellist']:
+                    # 2D type parameter
                     shape = (n_time, n_lat, n_lon)
                     n_dim_parameter = 2
                 elif self.era_setup["temperature"]['levellist'].count("/") + 1 == n_lev:
@@ -542,11 +556,16 @@ class ECMWF_download(object):
 
                 for name, obj in meta_data:
                     era_h5_file.create_array(era_h5, name=name, obj=obj)
-                data = era_h5_file.create_carray(era_h5, name=parameter, title=self.era_setup[parameter]["Name"],
-                                                 atom=at, shape=shape, filters=fl)
+                if parameter in ["fc_T2M", "fc_O3", "fc_SLP"]:
+                    naming = {"fc_T2M": "2 metre temperature", "fc_O3": "GEMS Total column ozone",
+                              "fc_SLP": "Surface pressure"}
+                    title = naming[parameter]
+                else:
+                    title = self.era_setup[parameter]["Name"]
+                data = era_h5_file.create_carray(era_h5, name=parameter, title=title, atom=at, shape=shape, filters=fl)
                 data[:, :, :] = np.NaN
 
-                for ii, grb in enumerate(era_data_file.select(name=self.era_setup[parameter]["Name"], year=self.year)):
+                for ii, grb in enumerate(era_data_file.select(name=title, year=self.year)):
                     tind = self.time_to_ind(self.time_to_julian_day(grb["year"], grb["month"], grb["day"]),
                                             {0: 0, 600: 1, 1200: 2, 1800: 3}[grb["time"]])
                     if tp == "reanalysis":
@@ -650,15 +669,17 @@ class ECMWF_download(object):
 
 def test():
     from ecmwfapi import ECMWFDataServer
-    self = ECMWF_download(server=ECMWFDataServer(), max_step=24)
-    if False:
-        self.retrieve_grib(parameter="surface_pressure", year=2013, month=5, day_start=1, day_end=2,
-                           output_file_name="./dummy_1.grib")
-        self.retrieve_hdf5(parameter="surface_pressure", year=2013, month_start=5, month_end=6, day_start=30, day_end=2,
-                           output_file_name="./dummy_2.h5")
+    import cdsapi
+
+    download_obj = ECMWF_download(server=ECMWFDataServer(), client=cdsapi.Client(), max_step=24)
+
     if True:
-        self.retrieve_hdf5(parameter="total_AOT_550nm", year=2016, month=5, day_start=17, day_end=17,
-                           output_file_name="./dummy_3.h5")
+        # test ECMWF data server download
+        download_obj.retrieve_hdf5(parameter="fc_total_AOT_550nm", year=2016, month=5, day_start=17, day_end=17,
+                                   output_file_name="./dummy_3.h5")
+        # test atmosphere data store download using the cdsapi client
+        download_obj.retrieve_hdf5(parameter="fc_SLP", year=2016, month=5, day_start=17, day_end=17,
+                                   output_file_name="./dummy_3.h5")
 
 
 def daterange(start_date, end_date):
@@ -720,14 +741,14 @@ class ECMWF_variable(object):
     def __call__(self, step, lons, lats, shape=None, order=3, dtype=np.float32, force_bounds=True):
         """
 
-        :param step: time in h
-        :param lons: 2D array of lon values: 0° - 359°  (correspond to: -180 - 180)
-                 prepare lons[-180:180] before: (360 + lons) % 360
-        :param lats: 2D array of lat values: -90° - 90°
-        :param shape: None or final shape, zoom is used for resampling
-        :param order: interpolation order of shape is not None
+        :param step:         time in h
+        :param lons:         2D array of lon values: 0° - 359°  (correspond to: -180 - 180)
+                             prepare lons[-180:180] before: (360 + lons) % 360
+        :param lats:         2D array of lat values: -90° - 90°
+        :param shape:        None or final shape, zoom is used for resampling
+        :param order:        interpolation order of shape is not None
         :param force_bounds: True / False, force step between bounds of the actually loaded file
-        :return: 2D array of variable
+        :return:             2D array of variable
         """
         from scipy.ndimage import zoom  # import here to avoid static TLS ImportError
 
@@ -752,13 +773,12 @@ class ECMWF_variable(object):
 
 
 def __download__(param):
-    from ecmwfapi.api import APIException
     # noinspection PyGlobalUndefined
     global download  # provided by initializer in multiprocessing mode
     try:
         download.retrieve_hdf5(**param)
         param["status"] = "OK"
-    except APIException as err:
+    except Exception as err:
         param["status"] = "error"
         param["error"] = repr(err)
     return param
@@ -783,9 +803,10 @@ def download_variables(ecmwf_variables, db_path, date_from, date_to, db_path_fun
 
     def initializer():
         from ecmwfapi import ECMWFDataServer
+        import cdsapi
         global download
-        # make ECMWF instance for data download
-        download = ECMWF_download(server=ECMWFDataServer(), max_step=max_step)
+        # make ECMWF and CDS instance for data download
+        download = ECMWF_download(server=ECMWFDataServer(), client=cdsapi.Client(), max_step=max_step)
 
     if db_path_function is None:
         db_path_function = ECMWF_download.db_path
@@ -807,5 +828,7 @@ def download_variables(ecmwf_variables, db_path, date_from, date_to, db_path_fun
     else:
         with Pool(processes=processes, initializer=initializer) as pool:
             pool.map(__download__, params)
+            pool.close()
+            pool.join()
 
     return params
