@@ -259,6 +259,8 @@ def interpol_lut(lut1, lut2, xnodes, nm_nodes, ndim, x_cell, vtest, intp_wvl):
     for ii in range(ndim):
         if vtest[ii] >= xnodes[:, ii].max():
             vtest[ii] = 0.99 * xnodes[-1, ii]
+        if vtest[ii] < xnodes[:, ii].min():
+            vtest[ii] = 1.01 * xnodes[0, ii]
 
         wh = np.where(vtest[ii] < xnodes[:, ii])[0]
         lim[0, ii] = wh[0] - 1
