@@ -70,7 +70,8 @@ def SLIC_segmentation(data_rad_all, n_pca, segs):
                   multichannel=True,
                   enforce_connectivity=True,
                   min_size_factor=0.5,
-                  max_size_factor=3)
+                  max_size_factor=3,
+                  start_label=0)
 
     lbl = np.unique(labels)
     segs = len(lbl)
@@ -81,7 +82,7 @@ def SLIC_segmentation(data_rad_all, n_pca, segs):
 def empirical_line_solution(X, rdn_subset, data_l2a_seg, rows, cols, bands, segs, labels, land_only, processes,
                             disable_progressbars):
     """
-    Apply empirical line solution to infer exact result of remaining spectra.
+    Empirical line solution to infer exact result of remaining spectra.
 
     :param X:                    flattened radiance data array
     :param rdn_subset:           radiance data subset
@@ -157,6 +158,8 @@ def empirical_line_solution(X, rdn_subset, data_l2a_seg, rows, cols, bands, segs
         with mp.get_context("fork").Pool(processes=processes, initializer=initializer, initargs=(globals(),
                                                                                                  _globs)) as pool:
             results = pool.map(_compute_coefficients_for_label, unique_labels)
+            pool.close()
+            pool.join()
 
     slopes, intercepts = np.empty([rows * cols, bands]), np.empty([rows * cols, bands])
 
@@ -174,6 +177,65 @@ def empirical_line_solution(X, rdn_subset, data_l2a_seg, rows, cols, bands, segs
     return reflectance
 
 
+def empirical_line_solution_snow(X, rdn_subset, data_l2a_seg, rows, cols, bands, segs, labels, processes):
+    """
+    Empirical line solution to infer exact result of remaining spectra, adapted to the estimation of snow and glacier
+    ice surface properties.
+
+    :param X:            flattened radiance data array
+    :param rdn_subset:   radiance data subset
+    :param data_l2a_seg: segmented surface reflectance
+    :param rows:         number of image rows
+    :param cols:         number of image columns
+    :param bands:        number of instrument bands
+    :param segs:         number of segments
+    :param labels:       array of segment labels for each pixel
+    :param processes:    number of CPUs for multiprocessing
+    :return:             extrapolated surface reflectance for each pixel
+    """
+    from scipy.spatial import KDTree  # import here to avoid static TLS ImportError
+
+    # First set up a matrix of locations, one per spectrum in both
+    # the reference set and the image cube.  We will simply use
+    # row and column indices for simplicity, rather than worrying
+    # about geographic or 3D locations
+    row_grid, col_grid = np.meshgrid(np.arange(rows), np.arange(cols))
+    locations = np.array([col_grid.flatten(), row_grid.flatten()]).T
+    locations_subset = np.zeros((segs, 2))
+
+    for i in range(segs):
+        locations_subset[i, :] = locations[labels.flat == i, :].mean(axis=0)
+    tree = KDTree(locations_subset)
+
+    # compute slopes and intercepts in the same shape like radiance and reflectance
+    unique_labels = np.unique(labels)
+    global _globs
+    _globs = dict(tree=tree, locs=locations_subset, k=15, rdn=rdn_subset, data=data_l2a_seg, bands=bands)
+
+    if platform.system() == "Windows" or processes == 1:
+        initializer(globals(), _globs)
+        results = []
+        for ii in tqdm(unique_labels):
+            (sl, ic) = _compute_coefficients_for_label(ii)
+            results.append((sl, ic))
+    else:
+        with mp.get_context("fork").Pool(processes=processes, initializer=initializer, initargs=(globals(),
+                                                                                                 _globs)) as pool:
+            results = pool.map(_compute_coefficients_for_label, unique_labels)
+            pool.close()
+            pool.join()
+
+    slopes, intercepts = np.empty([rows * cols, bands]), np.empty([rows * cols, bands])
+
+    for lbl, coeffs in zip(unique_labels, results):
+        slopes[labels.flat == lbl, :], intercepts[labels.flat == lbl, :] = coeffs
+
+    # finally compute reflectance
+    reflectance = X * slopes + intercepts
+
+    return reflectance
+
+
 _globs = dict()
 
 
