@@ -79,7 +79,7 @@ class GranuleDEM(object):
     def _to_target_resolution(self, arr):
         from scipy.ndimage import zoom  # import here to avoid static TLS ImportError
 
-        zf = self.sampling_to_shape[self.target_resolution] / np.array(arr.shape, dtype=np.float)
+        zf = self.sampling_to_shape[self.target_resolution] / np.array(arr.shape, dtype=float)
         if zf[0] == 1.0 and zf[1] == 1.0:
             return arr
         else:
